@@ -180,6 +180,12 @@ class _Scheduler(object):
             self.unsched_jobs = 0
             self._sched_jobs = {}
             self._sched_event = asyncoro.Event()
+            # once a _job is done (i.e., final result for it is
+            # received from node), it is added to done_jobs, so same
+            # object is not reused by Python (when a new job is
+            # submitted) until the result is sent back to client
+            # (otherwise, 'id' may be duplicate)
+            self.done_jobs = {}
             self.terminate = False
             self.sign = ''.join(hex(x)[2:] for x in os.urandom(20))
             self.auth_code = bytes(hashlib.sha1(bytes(self.sign + self.cluster_secret,
@@ -252,8 +258,6 @@ class _Scheduler(object):
                         pass
                     bc_sock.close()
             else:
-                if node_info['ip_addr'] in cluster._compute.nodes:
-                    continue
                 port = node_info['port']
                 if not port:
                     port = self.node_port
@@ -327,7 +331,8 @@ class _Scheduler(object):
                     if node.auth_code == info['auth_code']:
                         continue
                 def _get_pong(self, info, coro=None):
-                    sock = AsyncSocket(socket.socket(socket.AF_INET, socket.SOCK_STREAM))
+                    sock = AsyncSocket(socket.socket(socket.AF_INET, socket.SOCK_STREAM),
+                                       keyfile=self.node_keyfile, certfile=self.node_certfile)
                     sock.settimeout(2)
                     try:
                         pong_msg = {'ip_addr':info['scheduler_ip_addr'], 'port':self.port,
@@ -558,13 +563,16 @@ class _Scheduler(object):
         assert coro is not None
         logger.debug('Sending results for %s to %s, %s', uid, ip, port)
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        sock = AsyncSocket(sock)
+        sock = AsyncSocket(sock, keyfile=self.cluster_keyfile, certfile=self.cluster_certfile)
         sock.settimeout(5)
         try:
             yield sock.connect((ip, port))
             yield sock.send_msg(serialize(result))
             ack = yield sock.recv_msg()
             assert ack == b'ACK'
+            if result.status != DispyJob.ProvisionalResult:
+                if self.done_jobs.pop(uid, None) == None:
+                    logger.warning('job %s is not in "done"!' % uid)
         except:
             logger.debug(traceback.format_exc())
             f = os.path.join(self.dest_path_prefix, str(cid), '_dispy_job_reply_%s' % uid)
@@ -604,6 +612,7 @@ class _Scheduler(object):
                 cluster._pending_jobs -= 1
                 if cluster._pending_jobs == 0:
                     cluster.end_time = time.time()
+                self.done_jobs[_job.uid] = _job
                 Coro(self.send_job_result, _job.uid, cluster._compute.id,
                      cluster.client_ip_addr, cluster.client_job_result_port, reply)
 
@@ -1016,7 +1025,8 @@ class _Scheduler(object):
                 if node:
                     if node.auth_code == info['auth_code']:
                         raise StopIteration
-                sock = AsyncSocket(socket.socket(socket.AF_INET, socket.SOCK_STREAM))
+                sock = AsyncSocket(socket.socket(socket.AF_INET, socket.SOCK_STREAM),
+                                   keyfile=self.node_keyfile, certfile=self.node_certfile)
                 sock.settimeout(2)
                 try:
                     pong_msg = {'ip_addr':info['scheduler_ip_addr'], 'port':self.port,
@@ -1057,7 +1067,7 @@ class _Scheduler(object):
                     # logger.debug(traceback.format_exc())
                     pass
             else:
-                logger.warning('invalid job reply from %s:%s ignored' % addr)
+                logger.warning('invalid message from %s:%s ignored' % addr)
                 logger.debug(traceback.format_exc())
         conn.close()
 
@@ -1106,6 +1116,7 @@ class _Scheduler(object):
         job.start_time = reply.start_time
         job.end_time = reply.end_time
         if reply.status != DispyJob.ProvisionalResult:
+            self.done_jobs[_job.uid] = _job
             del self._sched_jobs[_job.uid]
             if reply.status == DispyJob.Finished:
                 node.jobs += 1
@@ -1150,7 +1161,7 @@ class _Scheduler(object):
                     node.ip_addr not in self._clusters[cluster_id]._compute.nodes) \
                    for cluster_id in node.clusters):
                 continue
-            logger.debug('load: %s, %s, %s' % (node.ip_addr, node.busy, node.cpus))
+            # logger.debug('load: %s, %s, %s' % (node.ip_addr, node.busy, node.cpus))
             if (load is None) or ((float(node.busy) / node.cpus) < load):
                 host = node
                 load = float(node.busy) / node.cpus
@@ -1247,6 +1258,7 @@ class _Scheduler(object):
         clusters = list(self._clusters.values())
         self._clusters = {}
         self._sched_jobs = {}
+        self.done_jobs = {}
         for cluster in clusters:
             compute = cluster._compute
             for node in compute.nodes.values():
@@ -1394,7 +1406,8 @@ if __name__ == '__main__':
     scheduler = _Scheduler(**config)
     while True:
         try:
-            sys.stdin.readline()
+            # sys.stdin.readline()
+            time.sleep(300)
         except KeyboardInterrupt:
             # TODO: terminate even if jobs are scheduled?
             logger.info('Interrupted; terminating')
